@@ -1,13 +1,15 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
+"""ffpuppet minidump parsing module"""
 
 from logging import getLogger
-from os import devnull, getenv, listdir
-from os.path import getmtime, isdir, join as pathjoin
+from os import getenv, scandir
+from os.path import getmtime, isdir
+from os.path import join as pathjoin
 from shutil import copy, copyfileobj
-from subprocess import call
-from tempfile import mkdtemp, TemporaryFile
+from subprocess import DEVNULL, call
+from tempfile import TemporaryFile, mkdtemp
 
 LOG = getLogger(__name__)
 
@@ -15,20 +17,27 @@ __author__ = "Tyson Smith"
 __all__ = ("process_minidumps",)
 
 
-class MinidumpParser(object):
+class MinidumpParser:  # pylint: disable=missing-docstring
     MDSW_BIN = "minidump_stackwalk"
     MDSW_MAX_STACK = 150
 
-    __slots__ = ("md_files", "symbols_path", "_include_raw", "_record_failures")
+    __slots__ = (
+        "md_files",
+        "symbols_path",
+        "_include_raw",
+        "_record_failures",
+        "_working_path",
+    )
 
-    def __init__(self, scan_path, record_failures=True):
+    def __init__(self, scan_path, record_failures=True, working_path=None):
         self.md_files = list()
-        for fname in listdir(scan_path):
-            if fname.endswith(".dmp"):
-                self.md_files.append(pathjoin(scan_path, fname))
+        for entry in scandir(scan_path):
+            if entry.name.endswith(".dmp"):
+                self.md_files.append(entry.path)
         self.symbols_path = None
         self._include_raw = getenv("FFP_DEBUG_MDSW") is not None
         self._record_failures = record_failures  # mdsw failure reporting
+        self._working_path = working_path
 
     def _call_mdsw(self, dump_file, out_fp, extra_flags=None):
         """Call minidump_stackwalk on a dmp file and collect output.
@@ -36,7 +45,7 @@ class MinidumpParser(object):
         Args:
             dump_file (str): Path to dmp file.
             out_fp (file): File to write output to.
-            extra_flags (list): Arguments to add to call to minidump_stackwalk.
+            extra_flags (list(str)): Arguments to pass to minidump_stackwalk.
 
         Returns:
             None
@@ -45,16 +54,18 @@ class MinidumpParser(object):
         if extra_flags:
             cmd += extra_flags
         cmd.append(dump_file)
-        cmd.append(self.symbols_path)
+        if self.symbols_path is not None:
+            cmd.append(self.symbols_path)
 
-        with TemporaryFile() as err_fp:
+        LOG.debug("calling %r", " ".join(cmd))
+        with TemporaryFile(dir=self._working_path, prefix="mdsw_stderr_") as err_fp:
             ret_val = call(cmd, stdout=out_fp, stderr=err_fp)
             if ret_val != 0:
                 # mdsw failed
                 LOG.warning("%r returned %r", " ".join(cmd), ret_val)
                 if self._record_failures:
                     # save the dmp file and the logs
-                    report_dir = mkdtemp(prefix="mdsw_err_")
+                    report_dir = mkdtemp(dir=self._working_path, prefix="mdsw_err_")
                     copy(dump_file, report_dir)
                     with open(pathjoin(report_dir, "mdsw_cmd.txt"), "wb") as log_fp:
                         log_fp.write((" ".join(cmd)).encode("ascii"))
@@ -64,7 +75,7 @@ class MinidumpParser(object):
                     out_fp.seek(0)
                     with open(pathjoin(report_dir, "mdsw_stdout.txt"), "wb") as log_fp:
                         copyfileobj(out_fp, log_fp, 0x10000)
-                    LOG.warning("mdsw failure can be found @ %r", report_dir)
+                    LOG.warning("mdsw logs can be found here %r", report_dir)
                     raise RuntimeError("MDSW Error")
         out_fp.seek(0)
 
@@ -78,8 +89,7 @@ class MinidumpParser(object):
         Returns:
             None
         """
-        LOG.debug("calling minidump_stackwalk on %s", dump_file)
-        with TemporaryFile() as out_fp:
+        with TemporaryFile(dir=self._working_path, prefix="mdsw_reg_") as out_fp:
             self._call_mdsw(dump_file, out_fp)
             found_registers = False
             for line in out_fp:  # pylint: disable=not-an-iterable
@@ -108,8 +118,7 @@ class MinidumpParser(object):
         Returns:
             None
         """
-        LOG.debug("calling minidump_stackwalk -m on %s", dump_file)
-        with TemporaryFile() as out_fp:
+        with TemporaryFile(dir=self._working_path, prefix="mdsw_stack_") as out_fp:
             self._call_mdsw(dump_file, out_fp, extra_flags=["-m"])
             if raw_fp is not None:
                 copyfileobj(out_fp, raw_fp, 0x10000)  # read in 64K chunks
@@ -137,7 +146,9 @@ class MinidumpParser(object):
                 log_fp.write(line)
                 line_count += 1
                 if line_count >= self.MDSW_MAX_STACK:
-                    LOG.warning("MDSW_MAX_STACK (%d) limit reached", self.MDSW_MAX_STACK)
+                    LOG.warning(
+                        "MDSW_MAX_STACK (%d) limit reached", self.MDSW_MAX_STACK
+                    )
                     log_fp.write(b"WARNING: Hit line output limit!")
                     break
 
@@ -158,7 +169,9 @@ class MinidumpParser(object):
             log_fp = cb_create_log("minidump_%02d" % count)
             self._read_registers(file_path, log_fp)
             # create log for raw mdsw stack output if needed
-            raw_fp = cb_create_log("raw_mdsw_%02d" % count) if self._include_raw else None
+            raw_fp = (
+                cb_create_log("raw_mdsw_%02d" % count) if self._include_raw else None
+            )
             self._read_stacktrace(file_path, log_fp, raw_fp)
             if log_fp.tell() < 1:
                 LOG.warning("minidump_stackwalk log was empty (minidump_%02d)", count)
@@ -172,18 +185,16 @@ class MinidumpParser(object):
             None
 
         Returns:
-            bool: True if minidump_stack walk is available otherwise False.
+            bool: True if minidump_stackwalk is available otherwise False.
         """
-        cmd = [cls.MDSW_BIN]
         try:
-            with open(devnull, "w") as null_fp:
-                call(cmd, stdout=null_fp, stderr=null_fp)
+            call([cls.MDSW_BIN], stdout=DEVNULL, stderr=DEVNULL)
         except OSError:
             return False
         return True
 
 
-def process_minidumps(scan_path, symbols_path, cb_create_log):
+def process_minidumps(scan_path, symbols_path, cb_create_log, working_path=None):
     """Scan for minidump (.dmp) files a in scan_path. If dumps are found they
     are parsed and new logs are added via the cb_create_log callback.
 
@@ -191,6 +202,7 @@ def process_minidumps(scan_path, symbols_path, cb_create_log):
         scan_path (str): Directory potentially containing minidump files.
         symbols_path (str): Directory containing symbols for the target binary.
         cb_create_log (callable): A callback to the add_log() of a PuppetLogger.
+        working_path (str): Used as base directory for temporary files.
 
     Returns:
         None
@@ -201,15 +213,17 @@ def process_minidumps(scan_path, symbols_path, cb_create_log):
     if not isdir(scan_path):
         LOG.debug("scan_path %r does not exist", scan_path)
         return
-    parser = MinidumpParser(scan_path)
+    parser = MinidumpParser(scan_path, working_path=working_path)
     if not parser.md_files:
         LOG.debug("scan_path %r did not contain '.dmp' files", scan_path)
         return
-    if not isdir(symbols_path):
-        LOG.warning("symbols_path not found: %r", symbols_path)
-        return
     if not parser.mdsw_available():
-        LOG.warning("Found a minidump, but can't process it without minidump_stackwalk."
-                    " See README.md for how to obtain it.")
+        LOG.warning(
+            "Found a minidump, but can't process it without minidump_stackwalk."
+            " See README.md for how to obtain it."
+        )
         return
+    if not isdir(symbols_path):
+        LOG.warning("Symbols path not found: %r", symbols_path)
+        symbols_path = None
     parser.collect_logs(cb_create_log, symbols_path)
